@@ -16,6 +16,7 @@ from toolspy import all_subclasses
 from schemalite.core import func_and_desc
 from sqlalchemy.orm import class_mapper
 from types import NoneType
+from sqlalchemy.dialects.mysql import MEDIUMTEXT
 
 
 def serialized_list(olist, rels_to_expand=[]):
@@ -46,7 +47,8 @@ def _set_fields_for_col(col_name, col, schema, forbidden, required):
                 sqltypes.String: (unicode, str),
                 sqltypes.Text: (unicode, str),
                 sqltypes.Boolean: (bool,),
-                sqltypes.Enum: (unicode, str)
+                sqltypes.Enum: (unicode, str),
+                MEDIUMTEXT: (unicode, str)
             }
             schema["fields"][col_name]["type"] = type_mapping[type(col.type)] + (NoneType, )
 
@@ -315,7 +317,7 @@ class DictizableMixin(object):
             rels_to_expand=rels_to_expand, rels_to_serialize=rels_to_serialize,
             key_modifications=key_modifications)
 
-    def todict_using_struct(self, dict_struct=None):
+    def todict_using_struct(self, dict_struct=None, dict_post_processors=None):
         """
             dict_struct:
             {
@@ -358,6 +360,10 @@ class DictizableMixin(object):
                         rel_obj, 'todict_using_struct') else rel_obj
             else:
                 result[rel] = None
+        if isinstance(dict_post_processors, list):
+            for dict_post_processor in dict_post_processors:
+                if callable(dict_post_processor):
+                    result = dict_post_processor(result, self)
         return result
 
 
@@ -367,7 +373,8 @@ class DictizableMixin(object):
                rels_to_serialize=None,
                group_listrels_by=None,
                key_modifications=None,
-               dict_struct=None):
+               dict_struct=None,
+               dict_post_processors=None):
 
         """Converts an instance to a dictionary form
 
@@ -409,7 +416,9 @@ class DictizableMixin(object):
         if dict_struct is None and self._autogenerate_dict_struct_if_none_:
             dict_struct = self.autogenerated_dict_structure()
         if dict_struct is not None:
-            return self.todict_using_struct(dict_struct)
+            return self.todict_using_struct(
+                dict_struct=dict_struct,
+                dict_post_processors=dict_post_processors)
         attrs_to_serialize = (
             self._attrs_to_serialize_ if attrs_to_serialize is None
             else attrs_to_serialize)
@@ -512,6 +521,11 @@ class DictizableMixin(object):
             if key in result:
                 result[mod_key] = result.pop(key)
 
+        if isinstance(dict_post_processors, list):
+            for dict_post_processor in dict_post_processors:
+                if callable(dict_post_processor):
+                    result = dict_post_processor(result, self)
+
         return result
 
     def serialize_attrs(self, *args):
@@ -531,12 +545,21 @@ class DictizableMixin(object):
         """
         # return dict([(a, getattr(self, a)) for a in args])
         cls = type(self)
-        return {
-            a: getattr(self, a)
-            for a in args
-            if hasattr(cls, a) and
-            a not in cls.attrs_forbidden_for_serialization()
-        }
+        result = {}
+        # result = {
+        #     a: getattr(self, a)
+        #     for a in args
+        #     if hasattr(cls, a) and
+        #     a not in cls.attrs_forbidden_for_serialization()
+        # }
+        for a in args:
+            if hasattr(cls, a) and a not in cls.attrs_forbidden_for_serialization():
+                val = getattr(self, a)
+                if is_list_like(val):
+                    result[a] = list(val)
+                else:
+                    result[a] = val
+        return result
         # return dict([(a, getattr(self, a)) for a in args if hasattr(cls, a) and a not in cls.attrs_forbidden_for_serialization()])
 
 
