@@ -650,6 +650,23 @@ class QueryableMixin(object):
             objs.append(cls.get_updated_or_new_obj(kwargs, keys))
         return objs
 
+    @classmethod
+    def update_or_build(cls, **kwargs):
+        keys = kwargs.pop('keys') if 'keys' in kwargs else []
+        filter_kwargs = subdict(kwargs, keys)
+        if filter_kwargs == {}:
+            obj = None
+        else:
+            obj = cls.first(**filter_kwargs)
+        if obj is not None:
+            for key, value in kwargs.iteritems():
+                if (key not in keys and
+                        key not in cls._no_overwrite_):
+                    setattr(obj, key, value)
+        else:
+            obj = cls.build(**kwargs)
+        return obj
+
     @classmethod
     def update_or_create(cls, **kwargs):
         """Checks if an instance already exists by filtering with the
@@ -954,3 +971,7 @@ class QueryableMixin(object):
         for k, v in cls._preprocess_params(kwargs).items():
             setattr(model, k, v)
         return model
+
+    @classmethod
+    def buckets(cls, bucket_size=None):
+    	return cls.query.buckets(bucket_size=bucket_size)
\ No newline at end of file
